%{
/*                         MICRION SOFTWARE DEPT.
 *          Copyright (c) 1994 by Micrion Corp. All rights reserved.
 *
 * FILE NAME    : mvcl.l
 * AUTHOR       : R. Mello
 * DATE WRITTEN : 12/12/94
 * VERSION      : 1.00
 * USAGE        : lexer for the mvcl programming language
 *
 *  This file contains a lex specification for the micrion ( M ) vacuum ( V )
 * command ( C ) language ( L ).
 */
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "mvcl.h"
#include <mvcl_int.h>

/* redefine flex/bison defaults. */
#define yylval  Mvcllval
#define yyerror Mvclerror

/* redefine YY_INPUT so we can switch the input source between strings and
 * streams.
 */
#undef YY_INPUT
#define YY_INPUT( b, r, ms )   ( r = MvclInputSource( b, ms ) )

/* define the min macro. */
#define  MIN(x,y)        (((x) < (y)) ? (x) : (y))

/* external reference to parser data structure. */
extern YYSTYPE  yylval;

/* local globals */
static int  lineno = 1;
static int  ParserModeToken;
static int  first_token = 1;
static int  InputSource = MVCL_INPUT_STREAM;
static char *InputString = (char *)NULL;
static char *InputStringLimit = (char *)NULL;
static char *InputStringPointer = (char *)NULL;

/* function prototypes */
int    MvclInputSource( char *, int );
void   yyerror( char * );
int    MvclGetLexerLine( void );
void   MvclSetParserMode( int );
int    MvclGetParserMode( void );
void   MvclSetSourceStream( FILE * );
int    Mvclwrap( void );
void   MvclSetInputSource( int );
void   MvclSetInputString( char * );
%}
 
integer		[0-9]+
dreal		([0-9]*"."[0-9]+)
ereal		([0-9]*"."[0-9]+[eE][+-]?[0-9]+)
real		{dreal}|{ereal}
nl		\n
string_term     \0
semi		;
ws		[ \t]+
comment		#.*
qstring		\"[^"\n]*["\n]
operators	\+|\-|\*|\/|\,|\(|\)
comparison      "=="|"!="|"<"|">"|"<="|">="
assignment      =
identifier      [A-Za-z][A-Za-z0-9_]*

%%

 /* Only return the mode token as the very first token read. */
 if( first_token )
   {
    first_token = 0;
    return( ParserModeToken );
   }

{nl}			{ /* Execute commands only for interactive stuff */
                          if( ParserModeToken != STREAM_START )
			    {
 			     ExecMvclCommand( MvclGetParserProgramList() );

                             /* Show the prompt only for the shell. */
                             if( InputSource == MVCL_STREAM )
     		               MvclShellPrompt();
			    }

                         /* Count lines... */
                         lineno++;
	 		}

{string_term}           { ;                     }

{semi}			{ return SEMICOLON;     }

{operators}		{ return yytext[0]; 	}

{comparison}		{
			 yylval.integer = MvclConvertComparisonToId( yytext );
			 return COMPARISON;
    			}

{assignment}            { return ASSIGNMENT;     }

{ws}|{comment}		{ ; 			}

{integer}		{
			 yylval.integer = atoi( yytext );
			 return INTEGER;
			}

{real}			{
			 yylval.real = atof( yytext );
			 return REAL;
			}

{qstring}		{
                         yylval.string = MvclStripQuotesOffString( yytext );
  			 return QSTRING;
			}

proc       		{ return PROC; 		}

end           		{ return END; 		}

if                      { return IF; 		}

then                    { return THEN;		}

else			{ return ELSE; 		}

endif   		{ return ENDIF;		}

for			{ return FOR;		}

to                      { return TO;            }

down[ \t]*to            { return DOWNTO;        }

endfor			{ return ENDFOR;	}

open			{ return OPEN; 		}

close			{ return CLOSE;		}

read			{ return READ; 		}

set			{ return SET; 		}

status			{ return STATUS;	}

wait			{ return WAIT; 		}

timeout			{ return TIMEOUT;	}

extend			{ return EXTEND;	}

retract			{ return RETRACT;	}

waitfor			{ return WAITFOR;	}

pumpon			{ return PUMPON;	}

pumpoff			{ return PUMPOFF;	}

error			{ return PERROR;	}

print                   { return PRINT;         }
 
return			{ return PRETURN;	}

v[ \t]*[0-9]{1,2} 	{ yyless(1); return V;    }

p[ \t]*[0-9]{1,2}	{ yyless(1); return P;    }

hcig[ \t]*[0-9]{1,2}	{ yyless(4); return HCIG; }

ccig[ \t]*[0-9]{1,2}	{ yyless(4); return CCIG; }

tc[ \t]*[0-9]{1,2}	{ yyless(2); return TC;   }

bt[ \t]*[0-9]{1,2}	{ yyless(2); return BT;   }

nz[ \t]*[0-9]{1,2}	{ yyless(2); return NZ;   }

mfc[ \t]*[0-9]{1,2}	{ yyless(3); return MFC;  }

pt[ \t]*[0-9]{1,2}	{ yyless(2); return PT;  }

help                    { return HELP;            }

{identifier}            {
                         if( MvclGetParserMode() == MVCL_STREAM )
                           {
			    yylval.string = strdup( yytext );
			    return IDENTIFIER;
                           }
                         else
                           {
                            yyerror( "Unknown-Command" );
			    return UNKNOWN;
                           }
			}

.                       { yyerror( "Unknown Character" ); return UNKNOWN; }

%%
 /****************************************************************************/
 int  MvclInputSource(
    char    *buf,                           /* pointer to working buffer     */
    int     max_size )                      /* maximum size of the buffer    */

   {
    /* MvclInputSource() -- This function is actually a replacement for the
     *     macro YY_INPUT. It will be used to distinguish between two input
     *     sources. Strings and file streams. If the input source is a stream
     *     then lexer will use the default behavior of YY_INPUT. If the source
     *     is a string. Then the lexer will read it's input from the string.
     *
     *     This function requires that the followin variables are properly
     *     initialized.
     *
     *       InputSource        -- Token source.
     *       InputString        -- Contains the string to parse.
     *       InputStringLimit   -- Pointer to the end of the string. 
     *       InputStringPointer -- Pointer to the current pos in input string.
     *
     *     RETURNS: the number of char's read.
     */
    int   n;
    int   result;
    int   c;

    /* Flex has a huge input buffer. More than most Mvcl scripts. So the 
     * performance hit taken here will not dramatically impact the performance
     * of the lexer.
     */
    if( InputSource == MVCL_INPUT_STRING )
      {
       n = MIN( max_size, InputStringLimit - InputStringPointer );

       /* If there are more char's to read. */
       if( n > 0 )
         {
          memcpy( buf, InputStringPointer, n );
          InputStringPointer += n;
         }
      
       /* just for the common return point. */
       result = n;
      }
    else
      {
       if( yy_current_buffer->yy_is_interactive )
         {
          c = getc( yyin );
          result = c == EOF ? 0 : 1;
          buf[0] = (char) c;
         }
       else if( ( ( result = fread( buf, 1, max_size, yyin ) ) == 0 )
                    && ferror( yyin ) )
             yyerror( "input in flex scanner failed" );
      }

    return( result );
   }
 /****************************************************************************/
void  yyerror( char *s )

  {
   /* yyerror() -- This function will report any errors during scanning.
    *
    *     RETURNS: nothing.
    */

   if( MvclGetParserMode() == MVCL_SHELL )
     fprintf( stderr, "Unknown Command: <%s>\n", yytext );
   else
     MvclParserError( "%s at token '%s'\n", s, yytext );
      
   return;
  }
/****************************************************************************/
int  MvclGetLexerLine( void )

  {
   /* MvclGetLexerLine( void ) -- This function will return the current
    *     line number that the lexer is working on.
    *
    *     RETURNS: the line number.
    */

   return( lineno );
  }
/****************************************************************************/
void  MvclSetParserMode(
   int      mode )

  {
   /* MvclSetParserMode() -- This function will set the desired type of
    *     parsing. One of three modes are selected, SHELL, STRING or STREAM.
    *     All that is really done is to set the first token that the lexer
    *     returns to a token that will select the proper grammar.
    *
    *     RETURNS: nothing.
    */

   /* Map request into the token value to return. */
   if( mode == MVCL_STREAM )
     ParserModeToken = STREAM_START;
   else if( mode == MVCL_SHELL )
     ParserModeToken = SHELL_START;
   else if( mode == MVCL_STRING )
     ParserModeToken = STRING_START;
   else
     ParserModeToken = MVCL_UNKNOWN;

   if( ParserModeToken != MVCL_UNKNOWN )
     {
      /* reset the line number counter and the first token flag. */
      lineno = 1;
      first_token = 1;
     }

   return;
  }
/****************************************************************************/
int  MvclGetParserMode( void )

  {
   /* MvclGetParserMode() -- This function will get the current parser mode.
    *
    *     RETURNS: The current mode.
    */
   int   mode;

   /* Map request into the token value to return. */
   if( ParserModeToken == STREAM_START )
     mode = MVCL_STREAM;
   else if( ParserModeToken = SHELL_START )
     mode = MVCL_SHELL;
   else if( ParserModeToken == STRING_START )
     mode = MVCL_STRING;
   else
     mode = MVCL_UNKNOWN;

   return( mode );
  }
/****************************************************************************/
void  MvclSetSourceStream(
   FILE    *source )

  {
   /* MvclSetSourceStream() -- This function will just tell the lexer where
    *     to get tokens from by assinging yyin to the source.
    *
    *     RETURNS: nothing.
    */

   yyin = source;
   return;
  }
/****************************************************************************/
int Mvclwrap( void )

  {
   /* Mvclwrap() -- This function will replace the standard yywrap.
    *
    *     RETURNS: 1 always -- there are no more files to read.
    */

   return( 1 );
  }
/****************************************************************************/
void  MvclSetInputSource(
   int     source )

  {
   /* MvclSetInputSource() -- This function will control the input source for
    *     the lexer. Input can be processed from either strings or streams.
    *
    *     RETURNS: nothing.
    */

   InputSource = source;

   /* for strings the first token must be reset. */
   if( InputSource == MVCL_INPUT_STRING )
     first_token = 1;
     
   return;
  }
/****************************************************************************/
void  MvclSetInputString(
   char  *string )

  {
   /* MvclSetInputString() -- This function will communicate the input string
    *     to this source file. It will also set the input string pointers to
    *     point to the appropriate positions in the string.
    *
    *     RETURNS: nothing.
    */

   if( InputString != (char *)NULL )
      free( InputString );

   if( string != (char *)NULL )
     {
      InputString = strdup( string );

      if( InputString != (char *)NULL )
        {
         /* Set the input source pointers. */
         InputStringPointer = InputString;
         InputStringLimit = InputString + strlen( InputString );
        }
      else
        fprintf(stderr,"Memory Allocation Error -- MvclSetInputString()\n");
     }
   else
     fprintf( stderr, "Null Input String In MvclSetInputString()\n" );
   
   return;
  }
/****************************************************************************/
/* finis */



