/*                         MICRION SOFTWARE DEPT.
 *          Copyright (c) 1995 by Micrion Corp. All rights reserved.
 *
 * FILE NAME    : mvcl_api.c
 * AUTHOR       : R. Mello
 * DATE WRITTEN : 7/17/95
 * VERSION      : 1.00
 * USAGE        : module of the mcvl parser
 *
 * This source file contains the applications programmer interface for mvcl.
 */
#include <stdio.h>                          /* for NULL, fprintf(), ect...   */
#include <mvcl_int.h>

/* function prototypes. */
int    MvclExecOpen( int );
int    MvclExecClose( int );
int    MvclExecStatus( int, int );
int    MvclExecPumpOn( int );
int    MvclExecPumpOff( int );
int    MvclExecExtend( int );
int    MvclExecRetract( int );
double MvclExecRead( int, int );
int    MvclExecSet( int, int, double );

/***************************************************************************/
int  MvclExecOpen(
   int     valve )                         /* number of the valve to open  */

  {
   /* MvclExecOpen() -- This function will attempt to open a valve.
    *
    *     RETURNS: 0 if the operation timed out.
    *              1 if the operation was a success.
    */
   int     ok = 0;
   char    cmd_str[128];

   sprintf( cmd_str, "open v%d\n", valve );
   ok = ExecMvclCommandString( cmd_str );

   return( ok );
  }
/***************************************************************************/
int  MvclExecClose(
   int     valve )                         /* number of the valve to close */

  {
   /* MvclExecClose() -- This function will attempt to close a valve.
    *
    *     RETURNS: 0 if the operation failed.
    *              1 if the operation was a success.
    */
   int     ok = 0;
   char    cmd_str[128];

   sprintf( cmd_str, "close v%d\n", valve );
   ok = ExecMvclCommandString( cmd_str );

   return( ok );
  }
/***************************************************************************/
int  MvclExecStatus(
   int     dev_class,                        /* class of device to use       */
   int     dev_id )                          /* specific device              */

  {
   /* MvclExecStatus() -- This function will attempt to chack the status
    *     of the requested device.
    *
    *     RETURNS: 0 if the operation failed.
    *              1 if the operation was a success.
    */
   int     ok = 0;
   char    cmd_str[128];

   sprintf( cmd_str,"status %s\n",MvclResolveDeviceName( dev_class, dev_id ) );
   ok = ExecMvclCommandString( cmd_str );

   return( ok );
  }
/***************************************************************************/
int  MvclExecPumpOn(
   int     pump )                         /* number of the pump to start    */

  {
   /* MvclExecPumpOn() -- This function will start a pump pumping.
    *
    *     RETURNS: 0 if the operation failed.
    *              1 if the operation was a success.
    */
   int     ok = 0;
   char    cmd_str[128];

   sprintf( cmd_str, "pumpon p%d\n", pump );
   ok = ExecMvclCommandString( cmd_str );

   return( ok );
  }
/***************************************************************************/
int  MvclExecPumpOff(
   int     pump )                         /* number of the pump to stop    */

  {
   /* MvclExecPumpOff() -- This function will stop a pump.
    *
    *     RETURNS: 0 if the operation failed.
    *              1 if the operation was a success.
    */
   int     ok = 0;
   char    cmd_str[128];

   sprintf( cmd_str, "pumpoff p%d\n", pump );
   ok = ExecMvclCommandString( cmd_str );

   return( ok );
  }
/***************************************************************************/
int    MvclExecExtend(
   int     nozzle )                         /* number of nozzle to extend  */

  {
   /* MvclExecExtend() -- This function will extend a nozzle.
    *
    *     RM@ -- NOTE: later I should add position information to nozzle stuff.
    *
    *     RETURNS: 0 if the operation failed.
    *              1 if the operation was a success.
    */
   int     ok = 0;
   char    cmd_str[128];

   sprintf( cmd_str, "extend nz%d\n", nozzle );
   ok = ExecMvclCommandString( cmd_str );

   return( ok );
  }
/***************************************************************************/
int    MvclExecRetract(
   int     nozzle )                         /* number of nozzle to extend  */

  {
   /* MvclExecRetract() -- This function will retract a nozzle.
    *
    *     RETURNS: 0 if the operation failed.
    *              1 if the operation was a success.
    */
   int     ok = 0;
   char    cmd_str[128];

   sprintf( cmd_str, "retract nz%d\n", nozzle );
   ok = ExecMvclCommandString( cmd_str );

   return( ok );
  }
/***************************************************************************/
double  MvclExecRead(
   int     dev_class,                      /* Class of device to read       */
   int     dev_id )                        /* id of device to read          */

  {
   /* MvclExecRead() -- This function will read the specified device.
    *
    *     RETURNS: The value read in from the device.
    */
   double  value = 0.0;
   char    cmd_str[128];
   int     ok = 0;

   sprintf( cmd_str,"read %s\n", MvclResolveDeviceName( dev_class, dev_id ) );
   ok = ExecMvclCommandString( cmd_str );

   return( value );
  }
/***************************************************************************/
int    MvclExecSet(
   int     dev_class,                     /* Class of device to read       */
   int     dev_id,                        /* id of device to read          */
   double  value )                        /* set it to this                */

  {
   /* MvclExecSet() -- This function will set the value on specified device.
    *
    *     RETURNS: 0 if the operation was a success.
    *              1 if the requested device was not valid.
    */
   int     ok = 0;
   char    cmd_str[128];

   sprintf( cmd_str,"set %s %f\n",
            MvclResolveDeviceName( dev_class, dev_id ), value );
   ok = ExecMvclCommandString( cmd_str );

   return( ok );
  }
/***************************************************************************/
/* finis */









