/*                         MICRION SOFTWARE DEPT.
 *          Copyright (c) 1994 by Micrion Corp. All rights reserved.
 *
 * FILE NAME    : mvcl_int.h
 * AUTHOR       : R. Mello
 * DATE WRITTEN : 3/21/95
 * VERSION      : 1.00
 * USAGE        : header for the mvcl interpreter.
 *
 */
#ifndef MVCL_INT_DEFINED

/* statement types. -- these equates must follow the same order as the 
 * dispatch table maintained by mvcl_int.c.
 */
#define  MVCL_OPEN     0
#define  MVCL_CLOSE    1
#define  MVCL_STATUS   2
#define  MVCL_SET      3
#define  MVCL_READ     4
#define  MVCL_TIMEOUT  5
#define  MVCL_PUMPON   6
#define  MVCL_PUMPOFF  7
#define  MVCL_EXTEND   8
#define  MVCL_RETRACT  9
#define  MVCL_WAIT_FOR 10
#define  MVCL_WAIT     11
#define  MVCL_IF       12
#define  MVCL_ELSE     13
#define  MVCL_ENDIF    14
#define  MVCL_FOR      15
#define  MVCL_ENDFOR   16
#define  MVCL_NULL     17
#define  MVCL_PRINT    18
#define  MVCL_RETURN   19

/* define the maximum allowable nested loops */
#define  MVCL_MAX_LOOP_NEST   25

/* define the different types of for statements. */
#define  MVCL_FOR_TO        0
#define  MVCL_FOR_DOWNTO    1

/* Define the different types of print statements. */
#define  MVCL_PRINT_STR     0
#define  MVCL_PRINT_IDENT   1
#define  MVCL_PRINT_REAL    2
#define  MVCL_PRINT_INT     3

/* Symbol table and stack equates. */
#define  MVCL_SYM_STRING    0
#define  MVCL_SYM_INT       1
#define  MVCL_SYM_DOUBLE    2
#define  MVCL_SYM_ADDRESS   3

/* File recording equates. */
#define  MVCL_RECORD_ NO    0
#define  MVCL_RECORD_YES    1

/* Constants used in general. */
#define  MVCL_VALVE_OPEN     0
#define  MVCL_VALVE_CLOSE    1
#define  MVCL_PUMP_START     2
#define  MVCL_PUMP_STOP      3
#define  MVCL_PUMP_STATUS    4
#define  MVCL_NOZZLE_EXTEND  5
#define  MVCL_NOZZLE_RETRACT 6
#define  MVCL_NOZZLE_STATUS  7

/* Constants used to determine the parser type. */
#define  MVCL_UNKNOWN      -1
#define  MVCL_STREAM        0
#define  MVCL_SHELL         1
#define  MVCL_STRING        2

/* Constants used to determine the input source for the lexer. */
#define  MVCL_INPUT_STREAM  0
#define  MVCL_INPUT_STRING  1

/* Constants used for the various device classes. */
#define  MVCL_VALVE_CLASS   0
#define  MVCL_PUMP_CLASS    1
#define  MVCL_HCIG_CLASS    2
#define  MVCL_CCIG_CLASS    3
#define  MVCL_BT_CLASS      4
#define  MVCL_TC_CLASS      5
#define  MVCL_NZ_CLASS      6
#define  MVCL_PT_CLASS      7
#define  MVCL_MFC_CLASS     8

/* Constants used to define the types of comparisons used. */
#define  MVCL_COMP_UNKNOWN  -1
#define  MVCL_EQUAL	    0
#define  MVCL_NOTEQUAL      1
#define  MVCL_LESSTHAN      2
#define  MVCL_GREATERTHAN   3
#define  MVCL_LESS_EQUAL    4
#define  MVCL_GREATER_EQUAL 5

/* Create a new type for pointer to a function returning an integer. */
#ifndef PFI_DEFINED
#define PFI_DEFINED
typedef int (*PFI)();
#endif

/* Create a new type for pointer to a function returning a void. */
#ifndef PFV_DEFINED
#define PFV_DEFINED
typedef void (*PFV)();
#endif

/* create a struct to represent an open statement. */
typedef struct MvclOpenStruct_tag
  {
   int        valve;                        /* id of the valve to open       */

  } MvclOpenStruct;

/* create a struct to represent an close statement. */
typedef struct MvclCloseStruct_tag
  {
   int        valve;                        /* id of the valve to close      */

  } MvclCloseStruct;

/* create a struct to represent an status statement. */
typedef struct MvclStatusStruct_tag
  {
   int        dev_class;                    /* class of device to get status */
   int        dev_id;                       /* specific device in the class  */
   void       *SymAddress;                  /* symbol table addr of identif  */

  } MvclStatusStruct;

/* create a struct to represent a set statement. */
typedef struct MvclSetStruct_tag
  {
   int        dev_class;                    /* class of device to get status */
   int        dev_id;                       /* specific device in the class  */
   double     dev_value;                    /* value to set the device at    */

  } MvclSetStruct;

/* create a struct to represent a read statement. */
typedef struct MvclReadStruct_tag
  {
   int        dev_class;                    /* class of device to read       */
   int        dev_id;                       /* specific device in the class  */
   double     dev_value;                    /* value read back from the dev  */
   void       *SymAddress;                  /* symbol table addr of identif  */

  } MvclReadStruct;

/* create a struct to represent a timeout statement. */
typedef struct MvclTimeoutStruct_tag
  {
   double     timeout;                      /* timeout value in seconds      */

  } MvclTimeoutStruct;

/* create a struct to represent a pump on statement. */
typedef struct MvclPumpOnStruct_tag
  {
   int     pump;                            /* id of the pump to start       */
   
  } MvclPumpOnStruct;

/* create a struct to represent a pump off statement. */
typedef struct MvclPumpOffStruct_tag
  {
   int     pump;                            /* id of the pump to stop        */

  } MvclPumpOffStruct;

/* create a struct to represent a extend statement. */
typedef struct MvclExtendStruct_tag
  {
   int     nozzle;                          /* id of the nozzle to extend    */

  } MvclExtendStruct;

/* create a struct to represent a retract statement. */
typedef struct MvclRetractStruct_tag
  {
   int     nozzle;                          /* id of the nozzle to retract   */

  } MvclRetractStruct;

/* create a struct to represent a waitfor statement. */
typedef struct MvclWaitForStruct_tag
  {
   int        dev_class;                    /* class of device to wait for   */
   int        dev_id;                       /* specific device in the class  */
   int        compare;                      /* enumerated comparison         */
   double     value;                        /* value to compare against      */

  } MvclWaitForStruct;

/* create a struct to represent a wait statement. */
typedef struct MvclWaitStruct_tag
  {
   double     wait_time;                    /* time to wait in seconds       */

  } MvclWaitStruct;

/* create a struct to represent an if statement */
typedef struct MvclIfStruct_tag
  {
   char       *ident_left;
   char       *ident_right;
   int        compare;
   void       *FalseAddress;

  } MvclIfStruct;

/* create a struct to represent an else statement */
typedef struct MvclElseStruct_tag
  {
   void       *EndifAddress;

  } MvclElseStruct;

/* create a struct to represent an if statement */
typedef struct MvclForStruct_tag
  {
   char       *control_ident;
   char       *target_ident;
   char       *start_ident;
   int        type;
   void       *EndForAddress;

  } MvclForStruct;

/* create a struct to represent an else statement */
typedef struct MvclEndForStruct_tag
  {
   char       *control_ident;
   char       *target_ident;
   int        type;
   void       *ForAddress;

  } MvclEndForStruct;

/* Create a new data type to represent a print item. -- I just want to mention
 * that this is not a statement in Mvcl this is just for the print statement.
 */
typedef struct MvclPrintItem_tag
  {
   char       *string;
   int        type;
   struct MvclPrintItem_tag   *next;  

  } MvclPrintItem;

/* Create a struct to represent a print statement. */
typedef struct MvclPrintStruct_tag
  {
   MvclPrintItem  *list;

  } MvclPrintStruct;

/* Create a struct to represent a print statement. */
typedef struct MvclReturnStruct_tag
  {
   int    return_value;

  } MvclReturnStruct;

/* create a new data type to represent a union of different mvcl statements.*/
typedef union
  {
   MvclOpenStruct      *OpenStatement;
   MvclCloseStruct     *CloseStatement;
   MvclStatusStruct    *StatusStatement;
   MvclSetStruct       *SetStatement;
   MvclReadStruct      *ReadStatement;
   MvclTimeoutStruct   *TimeoutStatement;
   MvclPumpOnStruct    *PumpOnStatement;
   MvclPumpOffStruct   *PumpOffStatement;
   MvclExtendStruct    *ExtendStatement;
   MvclRetractStruct   *RetractStatement;
   MvclWaitForStruct   *WaitForStatement;
   MvclWaitStruct      *WaitStatement;
   MvclIfStruct        *IfStatement;
   MvclElseStruct      *ElseStatement;
   MvclForStruct       *ForStatement;
   MvclEndForStruct    *EndForStatement;
   MvclPrintStruct     *PrintStatement;
   MvclReturnStruct    *ReturnStatement;

  } MvclStatementDef;

/* define a basic mvcl program node. */
typedef struct MvclProgramNode_tag
  {
   int                          StatementType;
   MvclStatementDef             def;
   struct MvclProgramNode_tag   *next;

  } MvclProgramNode;

/* Create a new union data type to represent all of the symbol values. */
typedef union
  {
   int       IntVal;
   double    DoubleVal;
   char      *StringVal;
   MvclProgramNode  *Address;

  } MvclSymbolType;

/* Create a new data type to represent a mvcl symbol. */
typedef struct  MvclSymbol_tag
  {
   char    *SymName;
   int     SymType;
   MvclSymbolType  def;

   struct MvclSymbol_tag   *next;
  } MvclSymbol;

/* Create a new data type to represent a mvcl symbol table. */
typedef struct  MvclSymbolTable_tag
  {
   int         SymCount;
   MvclSymbol  *head;
   MvclSymbol  *tail;

  } MvclSymbolTable;

/* Create a new data type to represent a run time stack entry. */
typedef struct  MvclRunTimeStackEntry_tag
  {
   int              ValueType;
   MvclSymbolType   Value;

   struct MvclRunTimeStackEntry_tag  *next;

  } MvclRunTimeStackEntry;

/* Create a new data type to represent the run time stack */
typedef struct MvclRunTimeStack_tag
  {
   int         StackSize;
   MvclRunTimeStackEntry   *top;

  } MvclRunTimeStack;

/* Now define a structure to represent a list of mvcl program nodes. */
typedef struct MvclProgramList_tag
  {
   /* Program list management. */
   MvclProgramNode      *head;
   MvclProgramNode      *tail;
   MvclProgramNode      *ip;

   /* Supporting cast of data structures. */
   MvclSymbolTable      *SymbolTable;
   MvclRunTimeStack     *RunTimeStack;

   /* global storage. */
   double               timeout_val;
   int                  return_value;

  } MvclProgramList;

/* function prototypes from mvcl.l */
char * ProcessQuotedString( char * );
void   yyerror( char * );
void   MvclSetParserMode( int );
int    MvclGetParserMode( void );
void   MvclSetSourceStream( FILE * );
int    MvclConvertComparisonToId( char * );
int    Mvclwrap( void );
void   MvclSetInputSource( int );
void   MvclSetInputString( char * );

/* function prototypes from mvcl.y */
MvclProgramList *   MvclGetParserProgramList( void );

/* function prototypes from mvcl_act.c */
int     MvclOpen( int );
int     MvclClose( int );
int     MvclStatus( int, int );
int     MvclPumpOn( int );
int     MvclPumpOff( int );
int     MvclSetDevice( int, int, double );
double  MvclReadDevice( int, int );
int     MvclExtendNozzle( int );
int     MvclRetractNozzle( int );
int     MvclWait( double );
int     MvclWaitFor( int, int, int, double );

/* function prototypes from mvcl_api.c */
int     MvclExecOpen( int );
int     MvclExecClose( int );
int     MvclExecStatus( int, int );
int     MvclExecPumpOn( int );
int     MvclExecPumpOff( int );
int     MvclExecExtend( int );
int     MvclExecRetract( int );
double  MvclExecRead( int, int );
int     MvclExecSet( int, int, double );

/* function prototypes from mvcl_bld.c */
int   BuildMvclOpenNode( MvclProgramList *, int );
int   BuildMvclCloseNode( MvclProgramList *, int );
int   BuildMvclStatusNode( MvclProgramList *, char *, int, int );
int   BuildMvclTimeoutNode( MvclProgramList *, double );
int   BuildMvclPumpOnNode( MvclProgramList *, int );
int   BuildMvclPumpOffNode( MvclProgramList *, int );
int   BuildMvclExtendNode( MvclProgramList *, int );
int   BuildMvclRetractNode( MvclProgramList *, int );
int   BuildMvclSetNode( MvclProgramList *, int, int, double );
int   BuildMvclReadNode( MvclProgramList *, char *, int, int );
int   BuildMvclWaitNode( MvclProgramList *, double );
int   BuildMvclWaitForNode( MvclProgramList *, int, int, int, double);
int   BuildMvclIfNode( MvclProgramList *, char *, int, char * );
int   BuildMvclElseNode( MvclProgramList * );
int   BuildMvclEndifNode( MvclProgramList * );
int   BuildMvclPrintNode( MvclProgramList *, MvclPrintItem * );
int   BuildMvclReturnNode( MvclProgramList *, int );

/* function prototypes from mvcl_err.c */

/* function prototypes from mvcl_lst.c */
MvclProgramList *  CreateMvclProgramList( void );
void     AddMvclProgramNode( MvclProgramList *, MvclProgramNode * );
void     DestroyMvclProgramList( MvclProgramList * );
void     DeleteMvclProgramList( MvclProgramList * );

/* function prototypes from mvcl_int.c */
int  MvclInterpreter( MvclProgramList *, int * );

/* function prototypes from mvcl_rec.c */
void  MvclOpenRecordFile( char *, char * );
void  MvclCloseRecordFile( void );
void  MvclSuspendRecording( void );
void  MvclResumeRecording( void );
void  MvclRecordProc( char * );
void  MvclRecordValve( int, int );
void  MvclRecordPump( int, int );
void  MvclRecordStatus( int, int );
void  MvclRecordExtend( int );
void  MvclRecordRetract( int );
void  MvclRecordRead( int, int );
void  MvclRecordSet( int, int, double );
void  MvclRecordWait( double );
void  MvclRecordWaitFor( int, int, int, double );
void  MvclRecordEnd( void );
void  MvclRecordComment( char * );
void  MvclRecordTimeout( double );
void  MvclRecordWait( double );
void  MvclRecordWaitFor( int, int, int, double );

/* function prototypes from mvcl_shl.c. */
int   MvclRun( char *, int * );
int   MvclShell( void );
void  MvclShellPrompt( void );
void  ExecMvclCommand( MvclProgramList * );
int   ExecMvclCommandString( char * );
MvclProgramList  *MvclCompile( char * );

/* function prototypes from mvcl_sym.c */
MvclSymbolTable * CreateMvclSymbolTable( void );
int    IsMvclSymbolInTable( MvclSymbolTable *, char * );
void * AddMvclSymbolToTable(MvclSymbolTable *,char *,int,MvclSymbolType *);
int    UpdateMvclSymbolInTable(MvclSymbolTable *, char *,int,MvclSymbolType *);
MvclSymbol * QueryMvclSymbolTable( MvclSymbolTable *, char * );
void   DestroyMvclSymbolTable( MvclSymbolTable * );

/* function prototypes from mvcl_stk.c */
MvclRunTimeStack *  CreateMvclRunTimeStack( void );
int      MvclRunTimeStackPush( MvclRunTimeStack *, MvclRunTimeStackEntry *);
MvclRunTimeStackEntry *  MvclRunTimeStackPop( MvclRunTimeStack * );
int      IsMvclRunTimeStackEmpty( MvclRunTimeStack * );
int      IsMvclRunTimeStackFull( MvclRunTimeStack * );
void     DestroyMvclRunTimeStack( MvclRunTimeStack * );
MvclRunTimeStackEntry *  CreateMvclRunTimeStackEntry( void );
void     DestroyMvclRunTimeStackEntry( MvclRunTimeStackEntry * );

/* function prototypes from mvcl_utl.c */
char *   MvclResolveDeviceName( int, int );
char *   MvclGenerateVariable( void );
char *   MvclResolveComparison( int );
int      MvclLinSearchForMatch( int, int *, int );
char *   MvclResolveDeviceStatus( int, int );
char *   MvclStripQuotesOffString( char * );
int      MvclConvertComparisonToId( char * );
char *   MvclCreateRealSymbol( MvclProgramList *, double );
char *   MvclCreateIntSymbol( MvclProgramList *, int );
char *   MvclStatementTypeToName( int );
void     MvclAddToPrintList( MvclPrintItem **, char *, int );

/* function prototypes from mvcl_vld.c */
void  MvclQueryVacuumSystem( char * );
int   MvclValidateDevice( int, int );
int   MvclValidateValve( int );
int   MvclValidatePump( int );
int   MvclValidateTC( int );
int   MvclValidateBT( int );
int   MvclValidateNozzle( int );
int   MvclValidatePT( int );
int   MvclValidateMFC( int );
int   MvclValidateHCIG( int );
int   MvclValidateCCIG( int );

/* function prototypes from mvcl_fil.c */
int *  MvclReadConfigOptions( FILE *, char *, int * );

#define MVCL_INT_DEFINED
#endif
/***************************************************************************/
/* finis */







