/*
 * JavaCallCmd.java
 *
 *	Implements the built-in "java::call" command.
 *
 * Copyright (c) 1997 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL
 * WARRANTIES.
 *
 * SCCS: @(#) JavaCallCmd.java 1.11 97/10/16 15:33:17
 *
 */

package tcl.lang;

/*
 * Implements the built-in "java::call" command.
 */

class JavaCallCmd implements Command {


/*----------------------------------------------------------------------
 *
 * cmdProc --
 *
 * 	This procedure is invoked to process the "java::call" Tcl
 * 	command. See the user documentation for details on what it
 * 	does.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	A standard Tcl result is stored in the interpreter.
 *
 *----------------------------------------------------------------------
 */

public void
cmdProc(
    Interp interp,			// Current interpreter.
    TclObject argv[])			// Argument list.
throws
    TclException			// A standard Tcl exception.
{
    boolean convert = true;
    int numArgs, startArg;

    if (argv.length < 3) {
	throw new TclNumArgsException(interp, 1, argv, 
		"class signature ?arg arg ...?");
    }

    /*
     * Get the class whose static method is to be invoked.
     */

    Class cls;
    try {
	cls = Invoke.getClassByName(argv[1].toString());
    } catch (ClassNotFoundException e1) {
	throw new TclException(interp, "unable to find class " +
		argv[1]);
    } catch (Exception e2) {
	throw new ReflectException(interp, e2);
    }

    numArgs = argv.length - 3;

    Object sig[] = ReflectObject.getSignature(interp, argv[2]);
    if (sig == null) {
	throw new TclException(interp,
		"bad signature: must specify a method name");
    }

    /*
     * Convert the TclObject's to Object's as accepted by Invoke.callMethod().
     */

    int i, j;
    Object args[] = new Object[numArgs];

    for (i = 0, j = 3; i < numArgs; i ++, j ++) {
	args[i]  = ReflectObject.wrapTclObject(interp, argv[j]);
    }

    /*
     * Call the method.
     */
     
    try {
	Object result[] = Invoke.callStaticMethod(cls, sig, args);
	ReflectObject.wrapJavaObject(interp, result);
    } catch (Exception ex) {
	throw new ReflectException(interp, ex);
    }
}

} // end JavaCallCmd
