/*
 * JavaInstanceofCmd.java
 *
 * Copyright (c) 1997 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL
 * WARRANTIES.
 *
 * SCCS: @(#) JavaInstanceofCmd.java 1.6 97/10/16 15:23:57
 */

package tcl.lang;

/**
 * This class implements the built-in "instanceof" command in Tcl.
 */

class JavaInstanceofCmd implements Command {
    /**
     * This procedure is invoked to process the "java::instanceof" Tcl
     * command.  See the user documentation for details on what it
     * does.
     *
     * @param interp the current interpreter.
     * @param argv command arguments.
     * @exception TclException if error occurs.
     */

    public void cmdProc(Interp interp, TclObject argv[]) throws TclException {
        if (argv.length != 3) {
	    throw new TclNumArgsException(interp, 1, argv, "object class");
	}

	Class cls = null;
	Object obj = null;

	obj = ReflectObject.get(interp, argv[1]);
	try {
	    cls = Invoke.getClassByName(argv[2].toString());
	} catch (ClassNotFoundException e1) {
	    throw new TclException(interp, "unknown class \"" + 
		    argv[2]  + "\"");
	}

	if (obj == null) {
	    interp.setResult(false);
	} else {
	    interp.setResult(cls.isInstance(obj));
	}
    }
}
