/*
 * JavaNewCmd.java
 *
 * Copyright (c) 1997 Sun Microsystems, Inc. All rights reserved.
 *
 * SCCS: @(#) JavaNewCmd.java 1.7 97/10/16 15:23:20
 *
 */

package tcl.lang;

/*
 * This class implements the built-in "java::new" command.
 */

class JavaNewCmd implements Command {

/*
 *----------------------------------------------------------------------
 *
 * cmdProc --
 *
 *	This procedure is invoked to process the "java::new" Tcl
 *	comamnd.  See the user documentation for details on what
 *	it does.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	A standard Tcl result is stored in the interpreter.
 *
 *----------------------------------------------------------------------
 */

public void
cmdProc(
    Interp interp,			// Current interpreter.
    TclObject argv[])			// Argument list.
throws
    TclException			// A standard Tcl exception.
{
    if (argv.length < 2) {
	throw new TclNumArgsException(interp, 1, argv,
		"signature ?arg arg ...?");
    }
    int numArgs = argv.length - 2;

    Object sig[] = ReflectObject.getSignature(interp, argv[1]);
    if (sig == null) {
	throw new TclException(interp,
		"bad signature: must specify a constructor name");
    }

    /*
     * Convert the TclObject's to Object's as accepted by
     * Invoke.callConstructor().
     */

    Object args[] = new Object[numArgs];
    for (int i = 0, j = 2; i < numArgs; i ++,j ++) {
	args[i]  = ReflectObject.wrapTclObject(interp, argv[j]);
    }

    /*
     * Call the constructor.
     */

    try {
	Object obj = Invoke.callConstructor(sig, args);
	interp.setResult(ReflectObject.newInstance(interp, obj));
    } catch (Exception ex) {
	throw new ReflectException(interp, ex);
    }
}

} // end JavaNewCmd

