/*
 * JavaPropCmd.java --
 *
 *	Implements the built-in "java::prop" command.
 *
 * Copyright (c) 1997 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL
 * WARRANTIES.
 *
 * SCCS: @(#) JavaPropCmd.java 1.4 97/10/16 14:53:21
 *
 */

package tcl.lang;

/*
 * This class implements the built-in "java::prop" command.
 */

class JavaPropCmd implements Command {


/*----------------------------------------------------------------------
 *
 * cmdProc --
 *
 * 	This procedure is invoked to process the "java::prop" Tcl
 * 	command.  See the user documentation for details on what they
 * 	do.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	A standard Tcl result is stored in the interpreter.
 *
 *----------------------------------------------------------------------
 */

public void
cmdProc(
    Interp interp,			// Current interpreter.
    TclObject argv[])			// Argument list.
throws
    TclException			// A standard Tcl exception.
{
    boolean query = false;
    TclObject queryName = null;
    Class cls;
    Object obj;

    if (argv.length < 3) {
	throw new TclNumArgsException(interp, 1, argv,
		"object property ?value property value ...?");
    }

    if (argv.length == 3) {
	query = true;
	queryName = argv[2];
    } else if ((argv.length % 2) != 0) {
	throw new TclNumArgsException(interp, 1, argv,
		"object property ?value property value ...?");
    }

    /*
     * In the java::prop command, only objects are allowed (classes
     * don't have properties.) TclException will be thrown by
     * ReflectObject.get if argv[1] is not a valid Java object.
     */

    obj = ReflectObject.get(interp, argv[1]);
    cls = obj.getClass();

    if (query) {
	/*
	 * Query the property.
	 */

	String name = queryName.toString();

	try {
	    Object result[] = Invoke.getProperty(cls, obj, name);
	    ReflectObject.wrapJavaObject(interp, result);
	} catch (Exception e2) {
	    throw new ReflectException(interp, e2);
	}
    } else {
	/*
	 * Set one or more properties.
	 */

	for (int i=2; i<argv.length; i+=2) {
	    String name = argv[i].toString();

	    try {
		Object jvalue = ReflectObject.wrapTclObject(interp,
			argv[i+1]);
		Invoke.setProperty(cls, obj, name, jvalue);
	    } catch (Exception e2) {
		throw new ReflectException(interp, e2);
	    }
	}
	interp.resetResult();
    }
}

} // end JavaPropCmd

