'\"
'\" Copyright (c) 1997 by Sun Microsystems, Inc.
'\"
'\" See the file "license.terms" for information on usage and redistribution
'\" of this file, and for a DISCLAIMER OF ALL WARRANTIES.
'\" 
'\" SCCS: @(#) java.n 1.11 97/10/21 12:59:34
'\" 
.so man.macros
.TH "Java" n 8.0 Tcl "Tcl Built-In Commands"
.BS
'\" Note:  do not modify the .SH NAME line immediately below!
.SH NAME
Java \- Java method invocation package
.SH SYNOPSIS
\fBpackage require java ?1.0?\fP
.sp
\fBjava::new \fIsignature\fR ?\fIarg arg ...\fR?
.sp
\fBjava::call \fIclass signature\fR ?\fIarg arg ...\fR?
.sp
\fBjava::field\fR \fIobjOrClass fieldSignature\fR ?\fIvalue fieldSignature value ...\fR?
.sp
\fBjava::instanceof\fR \fIjavaObj type\fR
.sp
\fBjava::prop\fR \fIjavaObj\fR \fIproperty\fR ?\fIvalue property value ...\fR?
.sp
\fBjava::info\fR \fIoption\fR ?\fIarg arg ...\fR?
.sp
\fBjava::null\fR
.sp
\fIjavaObj\fR \fIsignature\fR ?\fIarg arg ...\fR?
.sp
.BE

.SH DESCRIPTION
.PP
The \fBjava\fR package provides an interface for creating and
manipulating Java objects.  When the package is loaded, Tcl will use
an existing Java VM or initialize a new one as needed.  Note that
\fBtcl.lang\fR and the JDK 1.1 need to be on the class path.

.SH java::new
.PP
The \fBjava::new\fR command is used to create new instances of Java
objects from Tcl.  The \fIsignature\fR argument specifies which class
constructor to use for creating the object.  See SIGNATURE below for a
full description of how to specify the signature.  Additional
parameters to the java::new command are converted to Java objects or
primitive values and passed to the constructor.  The RETURN VALUES AND
EXCEPTIONS section below describes the result (and possible error
conditions) of the java::new command.

.SH java::call
.PP
The \fBjava::call\fR command is used to invoke public static methods
from Tcl.  The \fIclass\fR argument specifies the fully qualified name
of the declaring class of the method to invoke.  See the CLASS NAMES
below for a full description of fully qualified class names.  The
\fIsignature\fR argument specifies which class method to invoke (see
SIGNATURE below).  Additional parameters to the java::call command are
converted to Java objects or primitive types and passed to the method.
The RETURN VALUES AND EXCEPTIONS section below describes the result
(and possible error conditions) of the java::call command.

.SH java::field
.PP
The \fBjava::field\fR command is used to manipulate public fields from
Tcl.  The \fIobjOrClass\fR argument specifies either a fully qualified
name of the declaring class of the field to access, or an object
handle, as returned by \fBjava::new\fR, \fBjava::call\fR, or an object
command.  The \fIfieldSignature\fR argument specifies which field to
manipulate (see SIGNATURE below).  If an additional \fIvalue\fR
parameter exists, then the field will be set to \fIvalue\fR, otherwise
the current value of the field is returned.  Multiple fields may be
set via additional parameters by alternating field signatures and
values.

.SH java::instanceof
.PP
The \fBjava::instanceof\fR command is used to tell whether a Java
object is of a given type.  The \fIjavaObj\fR argument specifies an
object handle, as returned by \fBjava::new\fR, \fBjava::call\fR, or
the object command.  The \fItype\fR argument specifies a fully
qualified interface or class name (see CLASS NAMES below).  If the
\fItype\fR argument is a class name, \fBjava::instanceof\fR returns 1
if the \fIjavaObj\fR argument is an instance of \fItype\fR or an
instance of a subclass of \fItype\fR.  If the \fItype\fR argument is
an interface name, \fBjava::instanceof\fR returns true if the
\fIjavaObj\fR argument implements this interface. Otherwise,
\fBjava::instanceof\fR returns 0.

.SH java::prop
.PP
The \fBjava::prop\fR command is used to manipulate Java Bean
properties from Tcl.  The \fIjavaObj\fR argument specifies an object
handle, as returned by \fBjava::new\fR, \fBjava::call\fR, or an object
command.  The \fIproperty\fR argument specifies a Java Bean property
that has corresponding get and set methods.  If an additional
\fIvalue\fR parameter exists, then the property will be set to
\fIvalue\fR, otherwise the current value of the property is returned.
Multiple properties may be set via additional parameters by
alternating properties and values.

.SH java::info
.PP
The \fBjava::info\fR command provides introspection for Java classes,
object, and Beans.  The valid options for this command are:
.TP
\fBclass\fR \fIjavaObj\fR
Returns the class name of the specified Java object.
.TP
\fBfields\fR ?\fB-type\fR? ?\fB-static\fR? \fIobjOrClass\fR
Returns a list of fieldSignatures of public fields of the specified
class or Java object.  For shadowed superclass fields, the
fieldSignature is full.  For all other fields, the fieldSignature is
simple.  If the ?\fB-type\fR? flag is used, then each element of the
result list is a pair containing the data type and fieldSignature.  If
the \fB-static\fR flag is used, only static fields will appear in the
result list.  Otherwise, only non-static fields will appear in the
result list.
.TP
\fBmethods\fR ?\fB-type\fR? ?\fB-static\fR? \fIobjOrClass\fR
Returns a list of the full signatures of methods of the specified
class or Java object.  If the \fB-type\fR flag is used, the result is
a list of pairs of the method's return type and full signature.  If
the \fB-static\fR flag is used, only static methods will appear in the
result list.  Otherwise, only non-static methods will appear in the
result list.
.TP
\fBconstructors\fR \fIobjOrClass\fR
Returns a list of the full signatures of constructors of the specified
class or Java object.
.TP
\fBproperties\fR \fIobjOrClass\fR ?\fB-type\fR?
Returns a list of the names of Java Bean properties of the specified
class or Java object.  If the ?\fB-type\fR? flag is used, then each
element of the result list is a pair containing the data type and name
of the property.
.TP
\fBsuperclass\fR \fI\fIobjOrClass\fR
Returns the name of the immediate superclass of the specified Java
object or class.

.SH java::null
.PP
The \fBjava::null\fR command returns an object handle that represents
the "null" value in Java.  To check for null results from Java method
invocations, compare the methods' return values to the result of
\fBjava::null\fR.  The exact form of the return value of java::null is
not specified and is likely to change.

.SH OBJECT COMMAND
.PP
The object handle returned by \fBjava::new\fR, \fBjava::call\fR, or an
object command, is also the name of a Tcl command that can be used to
invoke public methods of that object from Tcl.  The \fIsignature\fR
argument specifies which class method to invoke (see SIGNATURE below).
Additional parameters to an object command are converted to Java
objects or primitive values and passed to the method.  The RETURN
VALUES AND EXCEPTIONS section below describes the result (and possible
error conditions) of the java::call command.

.SH CLASS NAMES
.PP
Any command which takes as input or returns a \fIclass name\fR,
\fIinterface name\fR, or primitive value expects a fully qualified
Java class name (e.g. java.awt.Button).  If a name is not fully
qualified, it is assumed to be in java.lang.*.

.SH SIGNATURE
.PP
A \fIsignature\fR is the string which specifies a class constructor,
method, or field, thereby distinguishing it from other constructors,
methods, and fields.  We will refer to signatures of fields as
\fIfieldSignatures\fR.  Any further mention of signatures refers to
those of both constructors and methods.

Two forms of signatures are accepted: simple and full.  The \fIsimple
signature\fR is a single element list containing the method or
constructor name.  If the simple signature is used to invoke an
overloaded method, then a method will be picked based on the number of
arguments passed to the command invoking the method.  The \fIfull
signature\fR is used to distinguish between two or more methods with
the same name and number of arguments.  The \fIfull signature\fR of a
method is a Tcl list containing the method name followed by the type
of each parameter of the method.

Two forms of \fIfieldSignatures\fR are accepted: simple and full.  A
\fIsimple fieldSignature\fR is a single element list containing the
filed name.  A \fIfull fieldSignature\fR is a Tcl list containing the
field name and the name of the class in which the field is declared.
The full form of \fIfieldSignature\fR is required to specify shadowed
fields of superclasses.

.SH "RETURN VALUES AND EXCEPTIONS"
.PP
When a constructor or other method is invoked by \fBjava::new\fR,
\fBjava::call\fR, or an object command, Tcl automatically converts the
result to a corresponding Tcl value.  If the type of the return is a
boolean or numeric type, it will be converted to an integer (or
floating-point) value.  If the result is a string, then the contents
of the string are returned.  For all other object types, a new Java
object handle is created and returned.  If the method throws an
exception, the string representation of the exception is returned as
an error and \fBerrorCode\fR will contain a list whose first element
is the string "JAVA" and whose second element is the object handle of
the exception object.

.SH "OBJECT GARBAGE COLLECTION"
.PP
The object handle associated with a Java object is considered to be an
object reference by the Java VM, ensuring that the Java object is not
garbage collected while Tcl is using it.  Tcl will release the object
handle when the last reference to the handle in a Tcl script goes
away.  A handle is considered to be active as long as at least one
\fBTcl_Obj\fR points to the handle.  In practice this means that Java
object handles must be stored in Tcl variables or passed as arguments
to other commands to keep them from being released.  Constructing a
Java object handle using \fBconcat\fR or some other string
manipulation command will produce a string that can be used where a
Java object handle is expected, but it will not count as a reference
to the object for garbage collection purposes.

.SH KEYWORDS
java, tcl

.SH SEE ALSO
\fBjava::load\fR

 
