'\"
'\" Copyright (c) 1997 by Sun Microsystems, Inc.
'\"
'\" See the file "license.terms" for information on usage and redistribution
'\" of this file, and for a DISCLAIMER OF ALL WARRANTIES.
'\" 
'\" SCCS: @(#) javaload.n 1.3 97/10/16 15:09:49
'\" 
.so man.macros
.TH "java::load" n 8.0 Tcl "Tcl Built-In Commands"
.BS
'\" Note:  do not modify the .SH NAME line immediately below!
.SH NAME
java::load \- Load a new extension into the current interpreter.
.SH SYNOPSIS
\fBjava::load \fIextensionName\fR
.sp
\fBjava::load ?\fI-classpath pathList\fR? \fIextensionName\fR
.sp
.BE

.SH DESCRIPTION
.PP
The java::load command loads extensions into the Java VM.  An Extension is a Java class that specifies a set of class files to load into the VM, as well as specifying how the Tcl interpreter should interact with these classes.  The primary use for this function is to load additional functionality (i.e. new commands) into the Tcl interpreter, but it may also be used to load classes into the VM that can be used by the Reflection API.  
.PP
The argument \fIextensionName\fR is the name of a class file that specifies which classes should be loaded as well as creating the binding between Tcl commands and their Java counterparts.  The class referenced by \fIextensionName\fR must be a subclass of \fItcl.lang.Extension\fR.  
.PP
Any class referenced by \fIextensionName\fR will also be loaded into the VM.  This includes classes used in new expressions as well as those whose instances are reported (i.e. the use of static methods.)  Due to the recursive nature of class loading, any class referenced must be located in CLASSPATH, env(TCL_CLASSPATH) or \fIpathList\fR.  If a class cannot be located in one of the three locations, a Tcl error is generated.
.PP
The \fIpathList\fR argument specifies additional paths to search in addition to the CLASSPATH and env(TCL_CLASSPATH) variables.  The env(TCL_CLASSPATH) can be set to contain a list of paths to search on every call to java::load.  If a path in \fIpathList\fR or env(TCL_CLASSPATH) is not an absolute path, it is assumed to be a relative path from the interpreters' current directory.  If a class, or any referenced class, cannot be resolved a Tcl error is generated.
.PP
.TP	
The process of resolving the class is done by searching, in order, one of the following locations.
.sp
1) The cache of previously loaded classes. 
.sp
2) The CLASSPATH list.
.sp
3) The \fIpathList\fR list.
.sp
4) The env(TCL_CLASSPATH) list, if it exists.
.sp
.LP
.PP
All JAR and ZIP files found in any of the paths will be searched in an attempt to find the class file.  The class will be automatically extracted from the JAR or ZIP file and loaded into the VM.
.PP
If the system class loader fails to load the class (step 2 above), a security check is done.  If the extension name begins with "java." or "tcl.", a Tcl error is generated.  This is done to assure that the class does not have access to extension-protected variables.
.PP

Currently it is not possible to unload or reload a extension.
.PP
.SH EXAMPLE
.PP
To load an extension stored in /proj/Jacl/Foo.class,
.DS
java::load -classpath /home/user/Jacl Foo
.DE
.SH BUGS
.PP
Currently '~' substitution is not performed on Unix systems.  The path 
must be absolute or relative from the current directory.  
