/*
 * java.h --
 *
 *	Declarations of structures and functions used to implement
 *	the native Java support in Tcl.
 *
 * Copyright (c) 1997 by Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * SCCS: @(#) java.h 1.16 97/10/21 14:20:45
 */

#ifndef _JAVA
#define _JAVA

#include <tcl.h>
#include <jni.h>

#if defined(__WIN32__)
#   define WIN32_LEAN_AND_MEAN
#   include <windows.h>
#   undef WIN32_LEAN_AND_MEAN

/*
 * VC++ has an alternate entry point called DllMain, so we need to rename
 * our entry point.
 */

#   if defined(_MSC_VER)
#	define EXPORT(a,b) __declspec(dllexport) a b
#	define DllEntryPoint DllMain
#   else
#	if defined(__BORLANDC__)
#	    define EXPORT(a,b) a _export b
#	else
#	    define EXPORT(a,b) a b
#	endif
#   endif
#else
#   define EXPORT(a,b) a b
#endif

/*
 * The following structure contains the state for the current Java VM.
 */

typedef struct JavaInfo {
    JavaVM *vm;			/* Java virtual machine. */
    JNIEnv *env;		/* Java run-time environment. */
    jclass InvocationTargetException;
    jmethodID getTargetException;
    jclass Object;
    jmethodID toString;
    jclass Invoke;
    jmethodID callMethod;
    jclass Class;
    jclass Field;
    jmethodID get;
    jclass String;
    jclass Boolean;
    jmethodID booleanValue;
    jclass Double;
    jclass Float;
    jclass Number;
    jmethodID intValue;
    jmethodID doubleValue;
    jclass Character;
    jclass Interp;
    jmethodID interpC;
    jfieldID interpPtr;
    jclass Command;
    jmethodID cmdProc;
    jclass TclObject;
    jmethodID preserve;
    jmethodID release;
    jmethodID getInternalRep;
    jclass TclException;
    jfieldID compCode;
    jclass CommandWithDispose;
    jmethodID disposeCmd;
    jclass CObject;
    jmethodID newCObjectInstance;
    jfieldID objPtr;
    jclass Extension;
    jmethodID loadOnDemand;
    jclass VarTrace;
    jmethodID traceProc;
    jclass Void;		/* java.lang.Void */
    jclass voidTYPE;		/* java.lang.Void.TYPE */
} JavaInfo;

extern JavaInfo java;

/*
 * The following variables represent the various Tcl_Obj types defined
 * by this package.
 */

extern Tcl_ObjType javaJRefType;
extern Tcl_ObjType javaTclObjectType;

/*
 * One of the following structures is created for every interpreter to
 * hold interpreter specific state information.
 */

typedef struct JavaInterpInfo {
    Tcl_HashTable *objTable;	/* Object name table for this interp. */
    jobject interpObj;		/* Java Interp object for this interp. */
} JavaInterpInfo;

/*
 * Declarations for functions shared across files.
 */

EXTERN void		JavaCatchException(JNIEnv *env, Tcl_Interp *interp,
			    Tcl_HashTable *tablePtr, jobject exception);
EXTERN Tcl_Interp *	JavaGetInterp(JNIEnv *env, jobject interpObj);
EXTERN Tcl_Obj *	JavaGetTclObj(JNIEnv *env, jobject object);
EXTERN jobject		JavaGetTclObject(JNIEnv *env, Tcl_Obj *objPtr,
			    int *isLocal);
EXTERN int		JavaGetObject(Tcl_Interp *interp,
			    Tcl_HashTable *hashPtr, Tcl_Obj *objPtr,
			    jobject *idPtr);
EXTERN void		JavaThrowTclException(JNIEnv *env, int result);

/*
 * Declarations for exported functions.
 */

EXTERN EXPORT(int,Java_Init) _ANSI_ARGS_((Tcl_Interp *interp));


#endif /* _JAVA */
