/*
 * BuildJava.java --
 *
 *	This Java class is used to invoke JAVAC.EXE on Windows
 *	platforms to compile Java sources files whose names are stored
 *	in a file. We need to do this because the standard Windows
 *	.BAT file doesn't seem to support this.
 *
 * Copyright (c) 1997 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL
 * WARRANTIES.
 *
 * SCCS: @(#) BuildJava.java 1.2 97/10/21 13:33:55
 *
 */

import java.io.*;

public class BuildJava {

public static void main(String argv[]) {
    if (argv.length != 2) {
	System.err.println("usage: java " + BuildJava.class.getName() +
		" argsFile javac");
	System.exit(-1);
    }

    FileInputStream fs;
    byte charArray[];

    File file = new File(argv[0]);
    if (!file.exists()) {
	System.err.println("no source to compile");
	System.exit(0);
    }

    /*
     * Read the contents of the argsFile into a String.
     */

    try {
	fs = new FileInputStream(argv[0]);
	charArray = new byte[fs.available()];
	fs.read(charArray);
    } catch (IOException e) {
	System.err.println(e);
	System.exit(-1);
	return;
    }

    StringBuffer sbuf = new StringBuffer();
    sbuf.append(argv[1]);
    sbuf.append(" ");

    /*
     * Sun JAVAC wants all file separators be "\" on Windows. If you
     * use "\" you'd get weird error messages.
     */

    String s = new String(charArray);
    for (int i=0; i<s.length(); i++) {
	if (s.charAt(i) == '/') {
	    sbuf.append('\\');
	} else if (s.charAt(i) == '\n') {
	    sbuf.append(" ");
	} else if (s.charAt(i) == '\r') {
	    sbuf.append(" ");
	} else {
	    sbuf.append(s.charAt(i));
	}
    }

    /*
     * Run JAVAC with the args.
     */

    try {
	Runtime.getRuntime().exec(sbuf.toString()).waitFor();
    } catch (Exception e) {
	System.err.println(e);
	System.exit(-1);
	return;
    }
}

}
